import request from 'supertest'
import { generateSimpleViteManifest, mockApp, mockConfig, mockFetch } from './util.js'
import { expect } from 'chai'
import { Response } from 'node-fetch'
import * as td from 'testdouble'

describe('Responses contain custom headers', function () {
  let fetchConfig
  let app

  before(async function () {
    mockConfig({ urls: ['http://ui-server/'] })
    mockFetch(fetchConfig = {
      'http://ui-server': {
        '/manifest.json': generateSimpleViteManifest({
          'example.js': {}
        }),
        '/example.js': () => new Response('this is example', { headers: { 'content-type': 'application/javascript' } }),
        '/meta.json': { name: 'sample-service', version: '1.0' }
      }
    })
    app = await mockApp()
  })

  after(function () {
    td.reset()
  })

  afterEach(function () {
    delete process.env.APP_VERSION
    delete process.env.BUILD_TIMESTAMP
    delete process.env.CI_COMMIT_SHA
  })

  it('has own metadata', async function () {
    process.env.APP_VERSION = '4.2'
    process.env.BUILD_TIMESTAMP = '0123456789'
    process.env.CI_COMMIT_SHA = '0123456789abcdef'

    const response = await request(app).get('/meta')
    expect(response.statusCode).to.equal(200)
    expect(response.body).to.deep.contain({
      id: 'ui-middleware',
      name: 'UI Middleware',
      buildDate: '0123456789',
      commitSha: '0123456789abcdef',
      version: '4.2'
    })
  })

  it('has metadata from another ui service if available', async function () {
    const response = await request(app).get('/meta')
    expect(response.statusCode).to.equal(200)
    expect(response.body).to.deep.contain({
      name: 'sample-service',
      version: '1.0'
    })
  })

  describe('without service avaible', function () {
    let prevConfig

    beforeEach(function () {
      prevConfig = fetchConfig['http://ui-server']
      delete fetchConfig['http://ui-server']
    })

    afterEach(function () {
      fetchConfig['http://ui-server'] = prevConfig
    })

    it('does not have metadata from ui service when unavailable', async function () {
      const response = await request(app).get('/meta')
      expect(response.statusCode).to.equal(200)
      expect(response.body).to.not.deep.contain({
        name: 'sample-service',
        version: '1.0'
      })
    })
  })
})
